@@ -83,11 +83,15 @@ try:
 except ImportError:
     from urllib import urlretrieve as _urlretrieve
 
-coco_url = "http://coco.gforge.inria.fr"
+coco_urls = ["https://coco.gforge.inria.fr/data-archive",  # original location
+             "https://numbbo.github.io/gforge/data-archive",  # new backup location
+             "https://numbbo.github.io/data-archive/data-archive",  # new location
+            ]
+coco_url = coco_urls[-1]  # may be reassigned if it doesn't work out
 cocopp_home = os.path.abspath(os.path.expanduser(os.path.join("~", ".cocopp")))
-cocopp_home_archives = os.path.join(cocopp_home, "data-archives")
 default_archive_location = os.path.join(cocopp_home, 'data-archives')
 default_definition_filename = 'coco_archive_definition.txt'
+cocopp_home_archives = default_archive_location
 listing_file_start = 'list_'
 listing_file_extension = '.txt'
 backup_last_filename = ''  # global variable to see whether and where a backup was made
@@ -184,12 +188,12 @@ def _str_to_list(str_or_list):
         return [str_or_list]
     raise ValueError(str_or_list)
 
-def _move_official_local_data():
+def _old_move_official_local_data():
     """move "official" archives folder to the generic standardized location once and for all"""
     src = os.path.join(cocopp_home, 'data-archive')
     dest = _url_to_folder_name(coco_url)
     if os.path.exists(src):
-        if not os.path.exists(os.path.join(dest, 'data-archive')):
+        if not os.path.exists(os.path.join(dest, 'data-archive')):  # obsolete
             _makedirs(dest)
             print("moving %s to %s" % (src, dest))
             _shutil.move(src, dest)
@@ -239,8 +243,8 @@ def _get_remote(url, target_folder=None, redownload=False):
     key = url
     url = official_archives.url(url) or url.rstrip('/')
     target_folder = target_folder or _url_to_folder_name(url)
-    if key in official_archives.names:
-        _move_official_local_data()  # once and for all
+    # if key in official_archives.names:  # old code
+    #     _move_official_local_data()  # once and for all
     if redownload or not os.path.exists(_definition_file_to_read(target_folder)):
         _makedirs(target_folder)
         _download_definitions(url, target_folder)
@@ -263,7 +267,7 @@ def _get_remote(url, target_folder=None, redownload=False):
             "or use the `update` method to re-download the remote definition file."
             % (_definition_file_to_read(target_folder), url))
         arch.remote_data_path = url
-    assert arch.remote_data_path == url  # check that url was in the definition file
+    assert arch.remote_data_path.replace('https', 'http') == url.replace('https', 'http')  # check that url was in the definition file
     return arch
 
 def get(url_or_folder=None):
@@ -287,13 +291,15 @@ def get(url_or_folder=None):
     and ``cocopp.archiving.ArchivesKnown()`` will show a list.
 
     >>> import cocopp
-    >>> url = 'http://lq-cma.gforge.inria.fr/data-archives/lq-gecco2019'
+    >>> url = 'https://cma-es.github.io/lq-cma/data-archives/lq-gecco2019'
     >>> arch = cocopp.archiving.get(url).update()  # downloads a 0.4KB definition file
     >>> len(arch)
     4
-    >>> assert arch.remote_data_path == url
+    >>> assert arch.remote_data_path.split('//', 1)[1] == url.split('//', 1)[1], (arch.remote_data_path, url)
  
     See `cocopp.archives` for "officially" available archives.
+
+    See also: `get_all`, `get_extended`.
     """
     if url_or_folder in (None, 'help'):
         raise ValueError(
@@ -373,7 +379,7 @@ def create(local_path):
                 and not fnlower.startswith('.')
                 and not default_definition_filename in filename
                 and not fnlower == 'readme'
-                and not fnlower.endswith(('.dat', '.rdat', '.tdat', '.info',
+                and not fnlower.endswith(('.git', '.dat', '.rdat', '.tdat', '.info',
                                           '.txt', '.md', '.py', '.ipynb', '.pdf'))
                 and not '.txt' in fnlower
                 ):
@@ -572,12 +578,12 @@ class COCODataArchive(_td.StrList):
         by repeatedly calling `get`. Elements of the `indices` list can
         be an index or a substring that matches one and only one name
         in the archive. If ``indices is None``, the results from the
-        last call to `find` are used. Download the data if necessary.
+        last call to `find` are used. Data are downloaded if necessary.
 
         See `find` or `cocopp.archiving.OfficialArchives` for how matching
         is determined.
 
-        See also `get`.
+        See also `get`, `get_extended`.
         """
         if indices is not None:  # TODO: just "if indices" should do?
             names = self.find(indices)
@@ -663,6 +669,10 @@ class COCODataArchive(_td.StrList):
                 if names[0] in self._redownload_if_changed:
                     self._download(names[0])
                     self.check_hash(full_name)
+                elif 11 < 3 and not input("\n\n  ** wrong hash, download {} again? [n=no, return=yes] **".format(names[0])).lower().startswith('n'):
+                    # seems to break tests?
+                    self._download(names[0])
+                    self.check_hash(full_name)
                 else:
                     raise
             try: self._redownload_if_changed.remove(names[0])
@@ -828,7 +838,8 @@ class COCODataArchive(_td.StrList):
                 '   %s\n'
                 'as it may be a partial or unsuccessful download.\n'
                 'A missing file will be downloaded again by `get`.\n'
-                'Alternatively, call `update` to update definitions\n'
+                'Alternatively, call `cocopp.archives.this-archive.update()`\n'
+                'of the respective data archive to update definitions\n'
                 'and checksums and allow for automatic re-downloads.\n'
                 'If this is not a remote archive consider to re-`create` it.'
                 '' % (name, self.local_data_path, str(self.remote_data_path),
@@ -854,7 +865,7 @@ class COCODataArchive(_td.StrList):
         As remote archives may grow or change, a common usecase may be
         
         >>> import cocopp.archiving as ac
-        >>> url = 'http://lq-cma.gforge.inria.fr/data-archives/lq-gecco2019'
+        >>> url = 'https://cma-es.github.io/lq-cma/data-archives/lq-gecco2019'
         >>> arch = ac.get(url).update()  # doctest:+SKIP
         
         For updating a local archive use::
@@ -1101,7 +1112,7 @@ class ListOfArchives(_td.StrList):
             f.write(_repr_definitions(self))
 
     def remote_update(self, name=None):
-        """join in the respective list from ``http://coco.gforge.inria.fr/data-archives``.
+        """join in the respective list from ``coco_url.rsplit('/')[0] + '/data-archives'``.
         
         Use `save` to save the joined entries.
         """
@@ -1250,15 +1261,15 @@ class OfficialArchives(object):
 
         Post-processing (2+)
           Using:
-            /.../.cocopp/data-archive/bbob/2009/BIPOP-CMA-ES_hansen_noiseless.tgz
-            /.../.cocopp/data-archive/bbob/2012/DE-AUTO_voglis_noiseless.tgz
-            /.../.cocopp/data-archive/bbob/2012/DE-BFGS_voglis_noiseless.tgz
-            /.../.cocopp/data-archive/bbob/2012/DE-ROLL_voglis_noiseless.tgz
-            /.../.cocopp/data-archive/bbob/2012/DE-SIMPLEX_voglis_noiseless.tgz
-            /.../.cocopp/data-archive/bbob/2012/DE_posik_noiseless.tgz
-            /.../.cocopp/data-archive/bbob/2012/DEAE_posik_noiseless.tgz
-            /.../.cocopp/data-archive/bbob/2012/DEb_posik_noiseless.tgz
-            /.../.cocopp/data-archive/bbob/2012/DEctpb_posik_noiseless.tgz
+            /.../.cocopp/data-.../bbob/2009/BIPOP-CMA-ES_hansen_noiseless.tgz
+            /.../.cocopp/data-.../bbob/2012/DE-AUTO_voglis_noiseless.tgz
+            /.../.cocopp/data-.../bbob/2012/DE-BFGS_voglis_noiseless.tgz
+            /.../.cocopp/data-.../bbob/2012/DE-ROLL_voglis_noiseless.tgz
+            /.../.cocopp/data-.../bbob/2012/DE-SIMPLEX_voglis_noiseless.tgz
+            /.../.cocopp/data-.../bbob/2012/DE_posik_noiseless.tgz
+            /.../.cocopp/data-.../bbob/2012/DEAE_posik_noiseless.tgz
+            /.../.cocopp/data-.../bbob/2012/DEb_posik_noiseless.tgz
+            /.../.cocopp/data-.../bbob/2012/DEctpb_posik_noiseless.tgz
 
         Post-processing (2+)
           loading data...
@@ -1275,14 +1286,17 @@ class OfficialArchives(object):
     >>> cocopp.main('-o myoutputfolder BIPOP! 2012/DE*')  # doctest:+SKIP
 
     """
-    def __init__(self):
+    def __init__(self, url=None):
         """all URLs and classes (optional) in one place.
         
         The archive names are identical with the last part of the URL. The only
         exception is made for `'all'`, which is removed to get the URL.
         """
         self.all = None  # only to prevent lint error in cocopp/__init__.py
-        self._base = coco_url + '/data-archive/'
+        if url is None:
+            url = coco_url
+        self._base = url.rstrip('/') + '/'
+        # self._base = coco_url + '/data-archive/'  # old way
         # TODO-decide: should this list better be initialized by a ListOfArchives file?
         #              (the same transition as before with the _all attribute in COCODataArchive)
         #              The code then only "hardcodes" the class name mapping?
@@ -1299,8 +1313,7 @@ class OfficialArchives(object):
     def add_archive(self, name):
         """Allow to use a new official archive.
         
-        The archive must exist as a subfolder of
-        http://coco.gforge.inria.fr/data-archive
+        The archive must exist as a subfolder of ``coco_url``.
         """
         self._list += [(self._base + name, None),]
         self.set_as_attributes_in()
@@ -1311,10 +1324,12 @@ class OfficialArchives(object):
         
         `target` is by default `self`.
         
-        Details: This method can only be called when the class names and the module
-        attribute `official_archives: OfficialArchive` (used in `get`) are
-        available. Depending on the implementation of `get`, it may download the
-        definition files on its first-ever call by/on any given user/machine.
+        Details: This method can only be called when the class names and
+        the module attribute `official_archives: OfficialArchive` (used in
+        `get`) are available. It creates new instances of the archives.
+        Depending on the implementation of `get`, it may download the
+        definition files on its first-ever call by/on any given
+        user/machine.
         """
         for name in self.names:
             if name not in except_for:
@@ -1322,6 +1337,21 @@ class OfficialArchives(object):
                         get(name).update() if update else get(name))
         return self
 
+    def link_as_attributes_in(self, target, except_for=('test',),
+                              update=False):
+        """Assign all archives as attribute of `target` except for ``'test'``.
+        
+        `target` is by default `self`.
+        
+        Details: This method can only be called when the class names and the module
+        attribute `official_archives: OfficialArchive` (used in `get`) are
+        available. It only creates links to the existing archives.
+        """
+        for name in self.names:
+            if name not in except_for:
+                name = name.replace('-', '_')
+                setattr(target, name, getattr(self, name))
+
     @property
     def names(self):
         """a list of valid key names"""
@@ -1351,13 +1381,63 @@ class OfficialArchives(object):
         return self._get(name, 1) or COCODataArchive
 
     def update_all(self):
-        self.set_as_attributes_in(update=True) 
+        """update archive definition files from their remote location.
 
-official_archives = OfficialArchives()
-# TODO-decide: when should we (try to) update these?
-official_archives.set_as_attributes_in()  # set "official" archives as attributes by suite name
+        The update is necessary to account for newly added data. This
+        method requires www connectivity with only a few KB of transmitted
+        data.
+        """
+        # self.set_as_attributes_in(update=True)
+        for name in self.names:
+            name = name.replace('-', '_')
+            try:
+                getattr(self, name).update()
+            except AttributeError:
+                if name != 'test':
+                    raise
 
-class _ArchivesOfficial(ListOfArchives):
+    def _make_folder_skeleton(self):
+        """workaround to avoid bailing when www is not reachable during
+
+        the very first import.
+        """
+        for url in [l[0] for l in self._list]:
+            path = _url_to_folder_name(url)
+            if path.endswith('/test'):
+                continue
+            if path.endswith('/all'):
+                path = path[:-4]
+                url = url[:-4]
+            _makedirs(path)  # may not be necessary?
+            df = _definition_file_to_read(path)
+            if os.path.exists(df):
+                warnings.warn('_make_folder_skeleton(): '
+                              'file "{}" exists'.format(df))
+            else:
+                with open(df, 'w') as f:
+                    f.write("[('_url_', '{}')]\n".format(url))
+
+# official_archives = OfficialArchives()
+for url in coco_urls[-1::-1]:  # loop over possible URLs until successful
+    official_archives = OfficialArchives(url)  # lazy init, does kinda nothing
+    coco_url = url
+    try:
+        # TODO-decide: when should we (try to) update/check these?
+        # The following `set_as_attributes_in` calls `cocopp.archiving.get(url)` and works if the
+        # connection was successful at least once (before or now)
+        official_archives.set_as_attributes_in()  # set "official" archives as attributes by suite name
+        break
+    except:  # (HTTPError, TimeoutError, URLError)
+        warnings.warn("failed to connect to " + url)
+else:
+    warnings.warn("Failed fo find workable URL or local folder for official archives."
+                  "\n After the www connection is restored, you may need to call"
+                  "\n `cocopp.archiving.official_archives.update_all()` to create"
+                  "\n valid definition files.")
+    official_archives._make_folder_skeleton()      
+    official_archives.set_as_attributes_in()
+
+class _old_ArchivesOfficial(ListOfArchives):
     """superseded by `OfficialArchives`
     
     Official COCO data archives.
@@ -1368,7 +1448,7 @@ class _ArchivesOfficial(ListOfArchives):
     __doc__ += ListOfArchives.__doc__
 
     listing_file = ListOfArchives._fullfile("official_archives")
-    search_folder = _abs_path(cocopp_home, "data-archive") 
+    search_folder = _abs_path(cocopp_home, "data-archive")  # obsolete
 
 class ArchivesLocal(ListOfArchives):
     """COCO data archives somewhere local on this machine.
@@ -1391,13 +1471,18 @@ class ArchivesKnown(ListOfArchives):
     search_folder = default_archive_location 
 
 class RemoteListOfArchives(_td.StrList):
-    """Elements of this list can be used directly in `cocopp.archiving.get`.
+    """Elements of this list can be used directly with `cocopp.archiving.get`.
 
     The only purpose of this list is to propose (or remind) known archive
-    locations to the user. For this purpose, the current listing file is
+    remote locations to the user. For this purpose, the current listing file is
     downloaded.
     """
-    location = coco_url + '/data-archives/'  # + 'list_known_archives.txt'
+    # was: location = coco_url + '/data-archives/'  # + 'list_known_archives.txt'
+    # a hack: remove deepest folder name in coco_url
+    location = coco_url.rstrip('/').rsplit('/', 1)[0]  # remove lowest folder name
+    if '//' not in location:
+        location = coco_url.rstrip('/')
+    location += '/data-archives/'  # + 'list_known_archives.txt'
 
     def __init__(self, name='known_archives'):
         super(RemoteListOfArchives, self).__init__(self._download(name))
